"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bignumber_js_1 = require("bignumber.js");
const ontology_ts_crypto_1 = require("ontology-ts-crypto");
const ontology_ts_test_1 = require("ontology-ts-test");
const configLoader_1 = require("../config/configLoader");
const punicaException_1 = require("../exception/punicaException");
const walletCli_1 = require("../wallet/walletCli");
// tslint:disable:no-console
class Assets {
    async transfer(projectDir, asset = 'ont', sender, to, walletFileName, amount, gasPrice, gasLimit, networkKey) {
        asset = asset.toLowerCase();
        if (asset !== 'ont' && asset !== 'ong') {
            throw punicaException_1.otherError('asset should be ont or ong');
        }
        const [wallet] = configLoader_1.loadWallet(projectDir, walletFileName);
        const account = configLoader_1.loadAccount(wallet, sender);
        const password = await walletCli_1.inputExistingPassword('Please input sender account password: ');
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const client = ontology_ts_test_1.initClient({ rpcAddress });
        const response = await ontology_ts_test_1.transfer({
            client,
            account,
            password,
            sender: account.address,
            to: ontology_ts_crypto_1.Address.fromBase58(to),
            amount,
            asset,
            wait: false,
            gasLimit,
            gasPrice
        });
        const result = response.result;
        if (result !== '') {
            console.log(`Transaction: ${result}`);
        }
        else {
            console.warn(`Transfer failed. Error: ${result}.`);
        }
    }
    async withdrawOng(projectDir, claimer, to, walletFileName, amount, gasPrice, gasLimit, networkKey) {
        const [wallet] = configLoader_1.loadWallet(projectDir, walletFileName);
        const account = configLoader_1.loadAccount(wallet, claimer);
        const password = await walletCli_1.inputExistingPassword('Please input claimer account password: ');
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const client = ontology_ts_test_1.initClient({ rpcAddress });
        const response = await ontology_ts_test_1.withdrawOng({
            client,
            account,
            asset: 'ong',
            password,
            sender: account.address,
            to: ontology_ts_crypto_1.Address.fromBase58(to),
            amount,
            wait: false,
            gasLimit,
            gasPrice
        });
        const result = response.result;
        if (result !== '') {
            console.log(`Transaction: ${result}`);
        }
        else {
            console.warn(`Withdraw ONG failed. Error: ${result}.`);
        }
    }
    async balanceOf(projectDir, asset = 'ong', address, networkKey) {
        asset = asset.toLowerCase();
        if (asset !== 'ont' && asset !== 'ong') {
            throw punicaException_1.otherError('asset should be ont or ong');
        }
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const client = new ontology_ts_test_1.RpcClient(rpcAddress);
        const response = await client.getBalance(ontology_ts_crypto_1.Address.fromBase58(address));
        if (response.error === 0 && response.result !== undefined) {
            if (asset === 'ong') {
                return new bignumber_js_1.default(response.result.ong).shiftedBy(-9).toString();
            }
            else {
                return response.result.ont;
            }
        }
        else {
            throw punicaException_1.otherError(response.desc);
        }
    }
    async unboundOng(projectDir, address, networkKey) {
        const rpcAddress = configLoader_1.loadNetwork(projectDir, networkKey);
        const client = new ontology_ts_test_1.RpcClient(rpcAddress);
        const response = await client.getUnboundOng(ontology_ts_crypto_1.Address.fromBase58(address));
        if (response.error === 0 && response.result !== undefined) {
            return new bignumber_js_1.default(response.result).shiftedBy(-9).toString();
        }
        else {
            throw punicaException_1.otherError(response.desc);
        }
    }
}
exports.Assets = Assets;
//# sourceMappingURL=assets.js.map